# pylint: disable=missing-module-docstring, missing-class-docstring, missing-function-docstring
import os
import pwd
import subprocess
from functools import total_ordering

from charmhelpers.core.hookenv import log
from distro_info import UbuntuDistroInfo


# class UnixUser(object):
class UnixUser:
    def __init__(self, username):
        self.username = username
        pwnam = pwd.getpwnam(username)
        self.uid = pwnam.pw_uid
        self.gid = pwnam.pw_gid

    def __enter__(self):
        log(
            "Raising UID to {uid} and GID to {gid}".format(
                uid=self.uid, gid=self.gid
            ),
            "INFO",
        )
        os.setresgid(self.gid, self.gid, os.getgid())
        os.setresuid(self.uid, self.uid, os.getuid())

    def __exit__(self, exc_type, exc_val, exc_tb):
        _, _, suid = os.getresuid()
        _, _, sgid = os.getresgid()
        log(
            "Restoring UID to {suid} and GID to {sgid}".format(
                suid=suid, sgid=sgid
            ),
            "INFO",
        )
        os.setresuid(suid, suid, suid)
        os.setresgid(sgid, sgid, sgid)


def install_autodep8(location):
    os.chdir(location)
    with UnixUser("ubuntu"):
        subprocess.check_call(["make"])
    subprocess.check_call(["make", "install"])


def pull(repository):
    """This will do a sort of git fetch origin && git reset --hard origin/master"""
    origin = [
        remote for remote in repository.remotes if remote.name == "origin"
    ][0]
    origin.fetch()
    remote_master_id = repository.lookup_reference(
        "refs/remotes/origin/master"
    ).target
    repository.checkout_tree(repository.get(remote_master_id))
    master_ref = repository.lookup_reference("refs/heads/master")
    master_ref.set_target(remote_master_id)
    repository.head.set_target(remote_master_id)


@total_ordering
# class UbuntuRelease(object):
class UbuntuRelease:
    all_releases = UbuntuDistroInfo().all

    def __init__(self, release):
        if release not in UbuntuRelease.all_releases:
            raise ValueError("Unknown release '{}'".format(release))
        self.release = release

    def __eq__(self, other):
        return UbuntuRelease.all_releases.index(
            self.release
        ) == UbuntuRelease.all_releases.index(other)

    def __le__(self, other):
        return UbuntuRelease.all_releases.index(
            self.release
        ) <= UbuntuRelease.all_releases.index(other)
